/*  Screem:  screem-window-menus.c
 *
 *  Handle ScreemWindow bonobo verbs
 *
 *  Copyright (C) 2001 David A Knight
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *  For contact information with the author of this source code please see
 *  the AUTHORS file.  If there is no AUTHORS file present then check the
 *  about box under the help menu for a contact address
 */
#include <config.h>

#include <string.h>

#include <bonobo/bonobo-ui-util.h>
#include <bonobo/bonobo-object.h>

#include <libgnome/gnome-i18n.h>
#include <libgnome/gnome-help.h>
#include <libgnome/gnome-exec.h>

#include <libgnomeui/gnome-about.h>
#include <libgnomeui/gnome-dialog-util.h>
#include <libgnomeui/gnome-file-entry.h>
#include <libgnomeui/gnome-entry.h>
#include <libgnomeui/gnome-stock-icons.h>

#include <libgnomevfs/gnome-vfs-mime-handlers.h>
#include <libgnomevfs/gnome-vfs-mime-utils.h>

#include <bonobo/bonobo-file-selector-util.h>

#include <glade/glade.h>

#include <gtk/gtkmain.h>
#include <gtk/gtkdialog.h>
#include <gtk/gtkcombo.h>
#include <gtk/gtkentry.h>

#include "screem-application.h"
#include "screem-window.h"
#include "screem-window-private.h"

#include "screem-site.h"
#include "screem-site-ui.h"
#include "screem-site-druid.h"

#include "screem-editor.h"
#include "screem-tree-view.h"

#include "screem-site-view.h"    /* don't like including this */

#include "screem-preferences.h"

#include "screem-page.h"
#include "pageUI.h"

#include "screem-todo.h"

#include "screem-cvs.h"

#include "support.h"

#include "fileops.h"
#include "editMenu.h"


static void
file_menu_save_document_as_callback( BonoboUIComponent *component,
				     gpointer user_data,
				     const char *verb );


static void
file_menu_new_window_callback( BonoboUIComponent *component,
			       gpointer user_data,
			       const char *verb )
{
	ScreemWindow *window;
	ScreemWindow *newwindow;
	ScreemApplication *app;
	ScreemSite *site;

	window = SCREEM_WINDOW( user_data );
	app = SCREEM_APPLICATION( window->application );

	site = screem_application_get_default_site( app );

	newwindow = screem_application_create_window( app );

	screem_window_set_current( newwindow, site );

	gtk_widget_show( GTK_WIDGET( newwindow ) );
}

static void
file_menu_new_site_callback( BonoboUIComponent *component,
			     gpointer user_data,
			     const char *verb )
{
	ScreemWindow *window;
	ScreemApplication *application;

	window = SCREEM_WINDOW( user_data );
	application = SCREEM_APPLICATION( window->application );

	screem_site_druid_begin( application, window );
}

static void
file_menu_new_document_callback( BonoboUIComponent *component,
				  gpointer user_data,
				  const char *verb )
{
	ScreemWindow *window;
	ScreemApplication *application;

	window = SCREEM_WINDOW( user_data );
	application = SCREEM_APPLICATION( window->application );

	screem_page_druid_begin( application, window );
}

static void
file_menu_new_blank_document_callback( BonoboUIComponent *component,
				       gpointer user_data,
				       const char *verb )
{
	ScreemWindow *window;
	ScreemSite *site;
	ScreemPage *page;

	window = SCREEM_WINDOW( user_data );
	site = screem_window_get_current( window );

	page = screem_page_new();
	screem_page_set_data( page, NULL );

	screem_site_add_page( site, page );
	screem_window_set_document( window, page );
}

static void
file_menu_new_site_template_callback( BonoboUIComponent *component,
				  gpointer user_data,
				  const char *verb )
{
	ScreemWindow *window;

	window = SCREEM_WINDOW( user_data );
	screem_site_create_template( window );
}


static void
file_menu_open_document_callback( BonoboUIComponent *component,
				  gpointer user_data,
				  const char *verb )
{
	ScreemWindow *window;
	ScreemApplication *application;
	ScreemSite *site;
	ScreemPage *page;
	gchar *filename;

	GList *list;

	window = SCREEM_WINDOW( user_data );
	application = SCREEM_APPLICATION( window->application );
	
	filename = bonobo_file_selector_open( NULL, TRUE,
					      _( "Open document..." ),
					      NULL, NULL );

	if( ! filename )
		return;

	list = application->loaded_sites;
	site = list->data;

	page = screem_page_new();

	screem_page_set_pathname( page, filename );

	/* add document to the site */
	if( ! screem_site_add_page( site, page ) ) {
		/* already open in the site, FIXME: what now? */
	}

	/* now display the document in the current view */
	screem_window_set_document( window, page );

	g_free( filename );
}

static void
file_menu_save_document_callback( BonoboUIComponent *component,
				  gpointer user_data,
				  const char *verb )
{
	ScreemWindow *window;
	ScreemApplication *application;
	ScreemPage *page;
	const gchar *filepath;

	window = SCREEM_WINDOW( user_data );
	application = SCREEM_APPLICATION( window->application );

	page = g_object_get_data( G_OBJECT( window ), "tab_popup" );
	if( ! page ) {
		page = screem_window_get_document( window );
	}

	/* Check if the page has a file path. */
	filepath = screem_page_get_pathname(page);
	if( filepath && strlen( filepath ) > 0 ) {
		/* Filename is normal, try a save straight away */
		if( ! screem_page_save( page ) ) {
			/* failed to save! */
			GtkWidget *widget;
			
			widget = gtk_message_dialog_new( GTK_WINDOW( window ),
							 GTK_DIALOG_MODAL,
							 GTK_MESSAGE_ERROR,
							 GTK_BUTTONS_OK,
							 _( "Failed to save page" ) );
			gtk_window_set_transient_for( GTK_WINDOW( widget ),
						      GTK_WINDOW( window ) );
			gtk_dialog_run( GTK_DIALOG( widget ) );
			gtk_widget_destroy( widget );
		}
	} else {
		/* If there's no file name, request one. */
		file_menu_save_document_as_callback(NULL, user_data, NULL);
	}
}

static void
file_menu_save_document_as_callback( BonoboUIComponent *component,
				     gpointer user_data,
				     const char *verb )
{
	ScreemWindow *window;
	ScreemApplication *application;
	ScreemEditor *editor;
	ScreemPage *page;

	window = SCREEM_WINDOW( user_data );
	application = SCREEM_APPLICATION( window->application );

	editor = SCREEM_EDITOR( window->details->editor );

	page = g_object_get_data( G_OBJECT( window ), "tab_popup" );
	if( ! page ) {
		page = screem_window_get_document( window );
	}

	if( ! screem_page_save_as( page, window ) ) {
		GtkWidget *widget;

		widget = gtk_message_dialog_new( GTK_WINDOW( window ),
						 GTK_DIALOG_MODAL,
						 GTK_MESSAGE_ERROR,
						 GTK_BUTTONS_OK,
						 _( "Failed to save page" ) );
		gtk_window_set_transient_for( GTK_WINDOW( widget ),
					      GTK_WINDOW( window ) );
		gtk_dialog_run( GTK_DIALOG( widget ) );
		gtk_widget_destroy( widget );
	}
}

static void
file_menu_close_document_callback( BonoboUIComponent *component,
				   gpointer user_data,
				   const char *verb )
{
	ScreemWindow *window;
	ScreemApplication *application;
	ScreemPage *page;
	ScreemSite *site;

	window = SCREEM_WINDOW( user_data );
	application = SCREEM_APPLICATION( window->application );

	site = screem_window_get_current( window );

	page = g_object_get_data( G_OBJECT( window ), "tab_popup" );
	if( ! page ) {
		page = screem_window_get_document( window );
	}
	screem_page_close_page( page, window );
	
	/* if no more pages are open set current doc to NULL */
	if( ! window->details->documents ) {
		screem_window_set_document( window, NULL );
	}

	if( screem_site_get_fake_flag( site ) ) {
		screem_site_remove_page( site, page );
		g_object_unref( page );
	}
}

static void
file_menu_open_site_callback( BonoboUIComponent *component,
			      gpointer user_data,
			      const char *verb )
{
	ScreemWindow *window;

	gchar *filename;

	window = SCREEM_WINDOW( user_data );

	filename = bonobo_file_selector_open( NULL, FALSE,
					 _( "Select Site..." ),
					 NULL, NULL );

	if( filename ) {
		screem_site_open_with_filename( window, filename );

		g_free( filename );
	}
}

static void
file_menu_save_all_callback( BonoboUIComponent *component,
			     gpointer user_data,
			     const char *verb )
{
	ScreemWindow *window;
	ScreemSite *site;
	
	window = SCREEM_WINDOW( user_data );
	site = screem_window_get_current( window );
	g_list_foreach( screem_site_get_pages( site ),
			(GFunc)screem_page_save, NULL );
}


static void
file_menu_close_site_callback( BonoboUIComponent *component,
			       gpointer user_data,
			       const char *verb )
{
	ScreemWindow *window;
	ScreemApplication *application;
	ScreemEditor *editor;
	ScreemSite *site;

	window = SCREEM_WINDOW( user_data );
	application = SCREEM_APPLICATION( window->application );

	editor = SCREEM_EDITOR( window->details->editor );

	site = screem_window_get_current( window );

	if( screem_site_get_fake_flag( site ) ) {
		g_assert( FALSE );
	} else {
		ScreemSite *next;
		screem_site_close_site( site );
	
		screem_application_close_site( application, site );
	
		next = screem_application_get_default_site( application );
		screem_window_set_current( window, next );

		g_object_unref( site );
	}  
}

static void
file_menu_print_callback( BonoboUIComponent  *component,
			  gpointer user_data,
			  const char *verb )
{
	ScreemWindow *window;

	window = SCREEM_WINDOW( user_data );

	screem_window_print( window, FALSE );
}

static void
file_menu_print_preview_callback( BonoboUIComponent  *component,
				  gpointer user_data,
				  const char *verb )
{

	ScreemWindow *window;

	window = SCREEM_WINDOW( user_data );

	screem_window_print( window, TRUE );
}

static void
file_menu_offline_callback( BonoboUIComponent *component,
			    gpointer user_data,
			    const char *verb )
{
	ScreemWindow *window;

	window = SCREEM_WINDOW( user_data );

	screem_window_toggle_online_status( window, GINT_TO_POINTER( 1 ) );
}

static void
file_menu_exit_application_callback( BonoboUIComponent *component,
				     gpointer user_data,
				     const char *verb )
{
	ScreemWindow *window;
	ScreemApplication *application;

	window = SCREEM_WINDOW( user_data );
	application = SCREEM_APPLICATION( window->application );
	gtk_main_quit();
}

static void
edit_menu_undo_callback( BonoboUIComponent *component,
			 gpointer user_data,
			 const char *verb )
{
	ScreemWindow *window;

	window = SCREEM_WINDOW( user_data );

	screem_view_undo( SCREEM_VIEW( window->details->current ) );
}

static void
edit_menu_redo_callback( BonoboUIComponent *component,
			 gpointer user_data,
			 const char *verb )
{
	ScreemWindow *window;

	window = SCREEM_WINDOW( user_data );

	screem_view_redo( SCREEM_VIEW( window->details->current ) );
}

static void
edit_menu_cut_callback( BonoboUIComponent *component,
			gpointer user_data,
			const char *verb )
{
	ScreemWindow *window;
	ScreemEditor *editor;

	window = SCREEM_WINDOW( user_data );
	
	editor = SCREEM_EDITOR( window->details->editor );
	
	if( GTK_WIDGET( editor ) == window->details->current ) {
		screem_editor_cut( editor );
	}
}

static void
edit_menu_copy_callback( BonoboUIComponent *component,
			gpointer user_data,
			const char *verb )
{
	ScreemWindow *window;
	ScreemEditor *editor;

	window = SCREEM_WINDOW( user_data );
	
	editor = SCREEM_EDITOR( window->details->editor );
	
	if( GTK_WIDGET( editor ) == window->details->current ) {
		screem_editor_copy( editor );
	}
}

static void
edit_menu_paste_callback( BonoboUIComponent *component,
			gpointer user_data,
			const char *verb )
{
	ScreemWindow *window;
	ScreemEditor *editor;

	window = SCREEM_WINDOW( user_data );
	
	editor = SCREEM_EDITOR( window->details->editor );
	
	if( GTK_WIDGET( editor ) == window->details->current ) {
		screem_editor_paste( editor );
	}
}

static void
edit_menu_paste_enc_callback( BonoboUIComponent *component,
			      gpointer user_data,
			      const char *verb )
{
	ScreemWindow *window;
	ScreemEditor *editor;

	window = SCREEM_WINDOW( user_data );
	
	editor = SCREEM_EDITOR( window->details->editor );
	
	if( GTK_WIDGET( editor ) == window->details->current ) {
		screem_editor_paste_encoded( editor );
	}
}

static void
edit_menu_clear_callback( BonoboUIComponent *component,
			  gpointer user_data,
			  const char *verb )
{
	ScreemWindow *window;
	ScreemEditor *editor;

	window = SCREEM_WINDOW( user_data );
	
	editor = SCREEM_EDITOR( window->details->editor );
	
	if( GTK_WIDGET( editor ) == window->details->current ) {
		screem_editor_clear_selection( editor );
	}
}

static void
edit_menu_select_all_callback( BonoboUIComponent *component,
			       gpointer user_data,
			       const char *verb )
{
	ScreemWindow *window;
	ScreemEditor *editor;

	window = SCREEM_WINDOW( user_data );
	
	editor = SCREEM_EDITOR( window->details->editor );
	
	if( GTK_WIDGET( editor ) == window->details->current ) {
		screem_editor_select_region( editor, 0, 0 );
	}
}

static void
edit_menu_select_context_callback( BonoboUIComponent *component,
				   gpointer user_data,
				   const char *verb )
{
	ScreemWindow *window;
	ScreemEditor *editor;
	ScreemTreeView *tree;

	guint pos;

	window = SCREEM_WINDOW( user_data );
	
	editor = SCREEM_EDITOR( window->details->editor );
	tree = SCREEM_TREE_VIEW( window->details->tree_view );

	pos = screem_editor_get_pos( editor );

	screem_tree_view_select_element_context( tree, pos,
						 NULL, NULL, FALSE );
}

static void
edit_menu_mark_editable_callback( BonoboUIComponent *component,
				   gpointer user_data,
				   const char *verb )
{
	ScreemWindow *window;
	ScreemWindowDetails *details;
	GladeXML *xml;
	GConfClient *client;
	gchar *glade_path;
	GtkWidget *widget;
	
	window = SCREEM_WINDOW( user_data );
	details = window->details;
	
	client = gconf_client_get_default();
	glade_path = gconf_client_get_string( client,
					      "/apps/screem/general/glade_path",
					      NULL );
	xml = glade_xml_new( glade_path, "mark_editable", NULL );
	g_free( glade_path );
	g_object_unref( client );
	
	widget = glade_xml_get_widget( xml, "mark_editable" );
	if( gtk_dialog_run( GTK_DIALOG( widget ) ) == GTK_RESPONSE_OK ) {
		const gchar *name;
		gchar *start;
		
		widget = glade_xml_get_widget( xml, "region_name" );
		name = gtk_entry_get_text( GTK_ENTRY( widget ) );
		
		start = g_strconcat( "<!-- #BeginEditable \"", name,
					"\" -->\n", NULL );
		
		screem_editor_insert_markup( SCREEM_EDITOR( details->editor ),
						start, 
						"<!-- #EndEditable -->\n" );
		g_free( start );
	}
	widget = glade_xml_get_widget( xml, "mark_editable" );
	gtk_widget_destroy( widget );
}

static void
edit_menu_bookmarks_callback( BonoboUIComponent *component,
			      gpointer user_data,
			      const char *verb )
{
	ScreemWindow *window;
	ScreemEditor *editor;
	ScreemPage *page;

	window = SCREEM_WINDOW( user_data );
	editor = SCREEM_EDITOR( window->details->editor );

	page = screem_window_get_document( window );

	screem_page_edit_bookmarks( window, editor, page );
}

static void
edit_menu_search_callback( BonoboUIComponent *component,
			   gpointer user_data,
			   const char *verb )
{
	ScreemWindow *window;

	window = SCREEM_WINDOW( user_data );

	if( ! strcmp( "Search", verb ) ) {
		screem_find( window );
	} else {
		screem_replace( window );
	}
}


static void
view_menu_editor_callback( BonoboUIComponent *component,
			   gpointer user_data,
			   const char *verb )
{
	ScreemWindow *window = SCREEM_WINDOW( user_data );

	screem_window_change_view( window, window->details->editor );
}

static void
view_menu_preview_callback( BonoboUIComponent *component,
			    gpointer user_data,
			    const char *verb )
{
	ScreemWindow *window = SCREEM_WINDOW( user_data );

	screem_window_change_view( window, window->details->preview );
}

static void
view_menu_linkview_callback( BonoboUIComponent *component,
			     gpointer user_data,
			     const char *verb )
{

	ScreemWindow *window = SCREEM_WINDOW( user_data );

	screem_window_change_view( window, window->details->link_view );
}

static void
view_menu_treeview_callback( BonoboUIComponent *component,
			     gpointer user_data,
			     const char *verb )
{
	ScreemWindow *window = SCREEM_WINDOW( user_data );

	screem_window_change_view( window, window->details->tree_view );
}

static void
insert_menu_doctype_callback( BonoboUIComponent *component,
			      gpointer user_data,
			      const char *verb )
{
	ScreemWindow *window = SCREEM_WINDOW( user_data );
	ScreemApplication *application;
	GladeXML *xml;
	gchar *glade_path;
	gchar *defaultdtd;
	GtkWidget *widget;
	gint button;
	
	application = SCREEM_APPLICATION( window->application );

	glade_path = gconf_client_get_string( application->client,
					      "/apps/screem/general/glade_path",
					      NULL );
	xml = glade_xml_new( glade_path, "insertdtd", NULL );
	g_free( glade_path );

	widget = glade_xml_get_widget( xml, "insertdtdcombo" );
	screem_dtd_fill_combo( application, widget );

	widget = GTK_COMBO( widget )->entry;
	defaultdtd = gconf_client_get_string(application->client,
					     "/apps/screem/editor/default_dtd",
					     NULL );
	gtk_entry_set_text( GTK_ENTRY( widget ), defaultdtd );
	g_free( defaultdtd );

	widget = glade_xml_get_widget( xml, "insertdtd" );

	gtk_window_set_transient_for( GTK_WINDOW( widget ),
				      GTK_WINDOW( window ) );
	button = gtk_dialog_run( GTK_DIALOG( widget ) );

	if( button == GTK_RESPONSE_OK ) {
		ScreemEditor *editor;
		const gchar *txt;

		editor = SCREEM_EDITOR( window->details->editor );

		widget = glade_xml_get_widget( xml, "insertdtdcombo" );
		widget = GTK_COMBO( widget )->entry;
		txt = gtk_entry_get_text( GTK_ENTRY( widget ) );

		if( ! strncmp( "<!DOCTYPE", txt, strlen( "<!DOCTYPE" ) ) ) {
			/* full doctype tag was entered, insert */
			screem_editor_insert( editor, 0, txt );
		} else {
			/* build doctype tag */
			gchar *tag;
			const gchar *systemid;
			const gchar *root;
			ScreemDTD *dtd;

			dtd = screem_application_add_dtd( application, txt,
							  NULL );
			root = screem_dtd_root_element_name( dtd );
			if( ! root ) {
				root = "unknown";
			}

			systemid = screem_dtd_get_system_id( application,
							     txt );
			tag = g_strconcat( "<!DOCTYPE ", root,
					   " PUBLIC \"", txt, 
					   "\" \"", systemid,
					   "\">", NULL );
			screem_editor_insert( editor, 0, tag );
			g_free( tag );
		}
	}

	widget = glade_xml_get_widget( xml, "insertdtd" );
	gtk_widget_destroy( widget );
}

static void
insert_menu_file_callback( BonoboUIComponent *component,
			   gpointer user_data,
			   const char *verb )
{
	ScreemWindow *window;
	ScreemWindowDetails *details;
	gchar *filename;

	window = SCREEM_WINDOW( user_data );
	details = window->details;
	
	filename = bonobo_file_selector_open( NULL, TRUE,
					      _( "Open document..." ),
					      NULL, NULL );

	if( filename && details->current == details->editor ) {
		screem_editor_insert_file( SCREEM_EDITOR( details->editor ),
					   filename );
	}

	g_free( filename );
}



static void
tool_menu_site_settings_callback( BonoboUIComponent *component,
				  gpointer user_data,
				  const char *verb )
{
	ScreemWindow *window;
	ScreemSite *site;

	window = SCREEM_WINDOW( user_data );

	site = screem_window_get_current( window );

	screem_site_settings_dialog( site, window );
}


static void
tool_menu_preferences_callback( BonoboUIComponent *component,
				gpointer user_data,
				const char *verb )
{
	ScreemWindow *window = SCREEM_WINDOW( user_data );
	ScreemApplication *application;

	application = SCREEM_APPLICATION( window->application );

	screem_preferences_edit( application );
}

/* FIXME: put some place sensible, doesn't belong in ScreemCVS though */
static gboolean screem_cvs_dialog( ScreemWindow *window,
				   gchar **log,
				   gchar **root,
				   gchar **module,
				   gchar **base,
				   gchar **flags )
{
	ScreemApplication *application;
	GConfClient *client;
       	GladeXML *xml;
	gchar *glade_path;
	GtkWidget *dialog;
	GtkWidget *widget;
	GtkTextBuffer *buffer;
	GtkTextIter it;
	GtkTextIter eit;
	const gchar *value;
	gint response;

	application = SCREEM_APPLICATION( window->application );
	client = application->client;
	glade_path = gconf_client_get_string( client,
					      "/apps/screem/general/glade_path",
					      NULL );
	xml = glade_xml_new( glade_path, "cvs_dialog", NULL );
	g_free( glade_path );

	dialog = glade_xml_get_widget( xml, "cvs_dialog" );

	if( log ) {
		widget = glade_xml_get_widget( xml, "logframe" );
		gtk_widget_show( widget );
	}

	if( root || module || base ) {
		widget = glade_xml_get_widget( xml, "checkoutframe" );
		gtk_widget_show( widget );
	}

	widget = glade_xml_get_widget( xml, "root" );
	gtk_widget_set_sensitive( widget, root != NULL );
	if( root ) {
		widget = gnome_entry_gtk_entry( GNOME_ENTRY( widget ) );
		gtk_entry_set_text( GTK_ENTRY( widget ), *root );
	}
	widget = glade_xml_get_widget( xml, "module" );
	gtk_widget_set_sensitive( widget, module != NULL );
	if( module ) {
		widget = gnome_entry_gtk_entry( GNOME_ENTRY( widget ) );
		gtk_entry_set_text( GTK_ENTRY( widget ), *module );
	}
	widget = glade_xml_get_widget( xml, "checkoutto" );
	gtk_widget_set_sensitive( widget, base != NULL );
	if( base ) {
		widget = gnome_file_entry_gtk_entry( GNOME_FILE_ENTRY(widget));
		gtk_entry_set_text( GTK_ENTRY( widget ), *base );
	}
	
	if( flags ) {
		widget = glade_xml_get_widget( xml, "flagframe" );
		gtk_widget_show( widget );

		widget = glade_xml_get_widget( xml, "cvsflags" );
		widget = gnome_entry_gtk_entry( GNOME_ENTRY( widget ) );
		gtk_entry_set_text( GTK_ENTRY( widget ), *flags );
	}

	gtk_window_set_transient_for( GTK_WINDOW( dialog ),
				      GTK_WINDOW( window ) );
	response = gtk_dialog_run( GTK_DIALOG( dialog ) );

	if( response == GTK_RESPONSE_OK ) {
		if( log ) {
			widget = glade_xml_get_widget( xml, "cvslog" );
			buffer = gtk_text_view_get_buffer( GTK_TEXT_VIEW( widget ) );
			gtk_text_buffer_get_start_iter( buffer, &it );
			gtk_text_buffer_get_end_iter( buffer, &eit );
			*log = gtk_text_buffer_get_text( buffer, &it, &eit,
							  TRUE );
		}

		if( root ) {
			widget = glade_xml_get_widget( xml, "root" );
			widget = gnome_entry_gtk_entry( GNOME_ENTRY(widget) );
			value = gtk_entry_get_text( GTK_ENTRY( widget ) );
			*root = g_strdup( value );
		}
		if( module ) {
			widget = glade_xml_get_widget( xml, "module" );
			widget = gnome_entry_gtk_entry( GNOME_ENTRY(widget) );
			value = gtk_entry_get_text( GTK_ENTRY( widget ) );
			*module = g_strdup( value );
		}
		if( base ) {
			widget = glade_xml_get_widget( xml, "checkoutto" );
			*base = gtk_editable_get_chars( GTK_EDITABLE(widget),
							0, -1 );
		}
		
		if( flags ) {
			widget = glade_xml_get_widget( xml, "cvsflags" );
			widget = gnome_entry_gtk_entry( GNOME_ENTRY( widget) );
			value = gtk_entry_get_text( GTK_ENTRY( widget ) );
			*flags = g_strdup( value );
		}
	}
	gtk_widget_destroy( dialog );

	return ( response == GTK_RESPONSE_OK );
}

static void
cvs_menu_checkout_site_callback( BonoboUIComponent *component,
				 gpointer user_data,
				 const char *verb )
{
	ScreemWindow *window;
	ScreemCVS *cvs;
	gchar *flags;
	gchar *module;
	gchar *root;
	gchar *base;

	window = SCREEM_WINDOW( user_data );
	cvs = window->details->cvs;

	flags = "";
	module = "";
	root = (gchar*)g_getenv( "CVSROOT" );
	if( ! root ) {
		root = "";
	}
	base = "";

	if( screem_cvs_dialog( window, NULL, &root, &module, &base, 
			       &flags ) ) {
		
		g_object_set( G_OBJECT( cvs ), 
			      "root", root,
			      "base", base, NULL );
		screem_cvs_checkout( cvs, flags, module );
		
		g_free( base );
		g_free( flags );
		g_free( module );
		g_free( root );
	}
}


static void
cvs_menu_update_site_callback( BonoboUIComponent *component,
			       gpointer user_data,
			       const char *verb )
{
	ScreemWindow *window;
	ScreemCVS *cvs;
	ScreemSite *site;

	gchar *root;
	gchar *module;
	gchar *base;
	gchar *flags;

	window = SCREEM_WINDOW( user_data );
	cvs = window->details->cvs;

	site = screem_window_get_current( window );

	root = (gchar*)screem_site_get_cvs_root( site );
	module = (gchar*)screem_site_get_name( site );
	base = (gchar*)screem_site_get_pathname( site );
	flags = "";

	if( screem_cvs_dialog( window, NULL, &root,
			       &module, &base, &flags ) ) {

		g_object_set( G_OBJECT( cvs ),
			      "root", root,
			      "base", base, NULL );

		screem_cvs_update( cvs, NULL, flags );

		g_free( module );
		g_free( base );
		g_free( flags );
	}
}


static void
cvs_menu_update_page_callback( BonoboUIComponent *component,
			       gpointer user_data,
			       const char *verb )
{
	ScreemWindow *window;
	ScreemCVS *cvs;
	ScreemSite *site;
	ScreemPage *page;

	const gchar *pathname;

	gchar *root;
	gchar *module;
	gchar *flags;
	gchar *base;
	
	gchar *temp;

	window = SCREEM_WINDOW( user_data );
	cvs = window->details->cvs;

	if( strcmp( "UpdateFile", verb ) ) {
		page = screem_window_get_document( window );
		
		pathname = screem_page_get_pathname( page );
	} else {
		GtkWidget *view;
		GtkTreeModel *model;
		GtkTreeSelection *selection;
		GtkTreeIter it;
		GValue value = { 0 };
		ScreemSiteViewNodeInfo *info;

		view = gtk_notebook_get_nth_page( GTK_NOTEBOOK( window->details->file_view ),
						  0 );
		view = gtk_bin_get_child( GTK_BIN( GTK_PANED(view)->child1 ) );
		model = gtk_tree_view_get_model( GTK_TREE_VIEW( view ) );
		selection = gtk_tree_view_get_selection( GTK_TREE_VIEW(view) );

		gtk_tree_selection_get_selected( selection, &model, &it );
		gtk_tree_model_get_value( model, &it, FILE_BROWSER_DATA_COL, &value );
		info = g_value_get_pointer( &value );
		if( info ) {
			pathname = info->fullname;
		} else {
			pathname = NULL;
		}
		g_value_unset( &value );
	}

	site = screem_window_get_current( window );
	if( ! screem_site_get_fake_flag( site ) ) {
		root = (gchar*)screem_site_get_cvs_root( site );
	       	module = (gchar*)screem_site_get_name( site );
		base = (gchar*)screem_site_get_pathname( site );
		temp = NULL;
	} else {
		root = (gchar*)g_getenv( "CVSROOT" );
		if( ! root ) {
			root = "";
		}
		module = "";
		temp = base = g_dirname( pathname );
	}
	flags = "";

	if( pathname && screem_cvs_dialog( window, NULL, 
					   &root, &module, &base,
					   &flags ) ) {

		g_object_set( G_OBJECT( cvs ),
			      "root", root,
			      "base", base, NULL );

		screem_cvs_update( cvs, pathname, flags );

		g_free( root );
		g_free( module );
		g_free( base );
		g_free( flags );
	} 

	if( temp ) {
		g_free( temp );
	}
}


static void
cvs_menu_import_site_callback( BonoboUIComponent *component,
			       gpointer user_data,
			       const char *verb )
{
	ScreemWindow *window;
	ScreemCVS *cvs;
	ScreemSite *site;

	gchar *log;
	gchar *root;
	gchar *module;
	gchar *base;
	gchar *flags;

	window = SCREEM_WINDOW( user_data );
	cvs = window->details->cvs;

	site = screem_window_get_current( window );

	log = "";
	root = (gchar*)screem_site_get_cvs_root( site );
	if( ! root ) {
		root = (gchar*)g_getenv( "CVSROOT" );
		if( ! root ) {
			root = "";
		}
	}
	module = (gchar*)screem_site_get_name( site );
	base = (gchar*)screem_site_get_pathname( site );
	flags = "";

	if( screem_cvs_dialog( window, &log, 
			       &root, &module, &base, &flags ) ) {

		g_object_set( G_OBJECT( cvs ),
			      "root", root,
			      "base", base, NULL );
	
		screem_cvs_import( cvs, flags, log, module );

		g_free( log );
		g_free( root );
		g_free( module );
		g_free( base );
		g_free( flags );
	}
}

static void
cvs_menu_commit_site_callback( BonoboUIComponent *component,
			       gpointer user_data,
			       const char *verb )
{
	ScreemWindow *window;
	ScreemCVS *cvs;
	ScreemSite *site;

	gchar *log;
	const gchar *root;
	const gchar *base;
	gchar *flags;

	window = SCREEM_WINDOW( user_data );
	cvs = window->details->cvs;

	site = screem_window_get_current( window );

	log = "";
	root = screem_site_get_cvs_root( site );
	if( ! root ) {
		root = g_getenv( "CVSROOT" );
		if( ! root ) {
			root = "";
		}
	}
	base = screem_site_get_pathname( site );
	flags = "";

	if( screem_cvs_dialog( window, &log, 
			       NULL, NULL, NULL, &flags ) ) {

		g_object_set( G_OBJECT( cvs ),
			      "root", root,
			      "base", base, NULL );
	
		screem_cvs_commit( cvs, NULL, flags, log );

		g_free( log );
		g_free( flags );
	}
}


static void
cvs_menu_commit_page_callback( BonoboUIComponent *component,
			       gpointer user_data,
			       const char *verb )
{
	ScreemWindow *window;
	ScreemCVS *cvs;
	ScreemSite *site;
	ScreemPage *page;

	const gchar *pathname;

	gchar *log;
	const gchar *root;
	const gchar *base;
	gchar *module;
	gchar *flags;

	gchar *temp;

	gchar **troot;
	gchar **tbase;
	gchar **tmodule;

	window = SCREEM_WINDOW( user_data );
	cvs = window->details->cvs;

	if( strcmp( "CommitFile", verb ) ) {
		page = screem_window_get_document( window );
		pathname = screem_page_get_pathname( page );
	} else {
		GtkWidget *view;
		GtkTreeModel *model;
		GtkTreeSelection *selection;
		GtkTreeIter it;
		GValue value = { 0 };
		ScreemSiteViewNodeInfo *info;

		view = gtk_notebook_get_nth_page( GTK_NOTEBOOK( window->details->file_view ),
						  0 );
		view = gtk_bin_get_child( GTK_BIN( GTK_PANED(view)->child1 ) );
		model = gtk_tree_view_get_model( GTK_TREE_VIEW( view ) );
		selection = gtk_tree_view_get_selection( GTK_TREE_VIEW(view) );

		gtk_tree_selection_get_selected( selection, &model, &it );
		gtk_tree_model_get_value( model, &it, FILE_BROWSER_DATA_COL, &value );
		info = g_value_get_pointer( &value );
		if( info ) {
			pathname = info->fullname;
		} else {
			pathname = NULL;
		}
		g_value_unset( &value );
	}

	site = screem_window_get_current( window );

	if( ! screem_site_get_fake_flag( site ) ) {
		log = "";
		root = screem_site_get_cvs_root( site );
		if( ! root ) {
			root = g_getenv( "CVSROOT" );
			if( ! root ) {
				root = "";
			}
		}
		base = screem_site_get_pathname( site );
		troot = NULL;
		tmodule = NULL;
		tbase = NULL;
		temp = NULL;
	} else {
		root = g_getenv( "CVSROOT" );
		if( ! root ) {
			root = "";
		}
		if( pathname ) {
			temp =  g_dirname( pathname );
		} else {
			temp = NULL;
		}
		base = (const gchar*)temp;

		module = "";
		tbase = &base;
		tmodule = &module;
		troot = &root;
	}
	flags = "";

	if( pathname && screem_cvs_dialog( window, &log, 
					   troot, tmodule, tbase,
					   &flags ) ) {

		g_object_set( G_OBJECT( cvs ),
			      "root", root,
			      "base", base, NULL );
	
		screem_cvs_commit( cvs, pathname, flags, log );

		g_free( log );
		g_free( flags );

		if( troot ) {
			g_free( *troot );
		}
		if( tmodule ) {
			g_free( *tmodule );
		}
		if( tbase ) {
			g_free( *tbase );
		}
	} 

	if( temp ) {
		g_free( temp );
	}
}

static void
cvs_menu_add_page_callback( BonoboUIComponent *component,
			    gpointer user_data,
			    const char *verb )
{
	ScreemWindow *window;
	ScreemCVS *cvs;
	ScreemSite *site;
	ScreemPage *page;

	const gchar *pathname;

	gchar *log;
	const gchar *root;
	const gchar *base;
	gchar *module;
	gchar *flags;

	gchar *temp;

	gchar **troot;
	gchar **tbase;
	gchar **tmodule;

	window = SCREEM_WINDOW( user_data );
	cvs = window->details->cvs;

	if( strcmp( "AddFile", verb ) ) {
		page = screem_window_get_document( window );
		pathname = screem_page_get_pathname( page );
	} else {
		GtkWidget *view;
		GtkTreeModel *model;
		GtkTreeSelection *selection;
		GtkTreeIter it;
		GValue value = { 0 };
		ScreemSiteViewNodeInfo *info;

		view = gtk_notebook_get_nth_page( GTK_NOTEBOOK( window->details->file_view ),
						  0 );
		view = gtk_bin_get_child( GTK_BIN( GTK_PANED(view)->child1 ) );
		model = gtk_tree_view_get_model( GTK_TREE_VIEW( view ) );
		selection = gtk_tree_view_get_selection( GTK_TREE_VIEW(view) );

		gtk_tree_selection_get_selected( selection, &model, &it );
		gtk_tree_model_get_value( model, &it, FILE_BROWSER_DATA_COL, &value );
		info = g_value_get_pointer( &value );
		if( info ) {
			pathname = info->fullname;
		} else {
			pathname = NULL;
		}
		g_value_unset( &value );
	}

	site = screem_window_get_current( window );

	if( ! screem_site_get_fake_flag( site ) ) {
		log = "";
		root = screem_site_get_cvs_root( site );
		if( ! root ) {
			root = g_getenv( "CVSROOT" );
			if( ! root ) {
				root = "";
			}
		}
		base = screem_site_get_pathname( site );
		troot = NULL;
		tmodule = NULL;
		tbase = NULL;
		temp = NULL;
	} else {
		root = g_getenv( "CVSROOT" );
		if( ! root ) {
			root = "";
		}
		if( pathname ) {
			temp =  g_dirname( pathname );
		} else {
			temp = NULL;
		}
		base = (const gchar*)temp;

		module = "";
		tbase = &base;
		tmodule = &module;
		troot = &root;
	}
	flags = "";

	if( pathname && screem_cvs_dialog( window, &log, 
					   troot, tmodule, tbase,
					   &flags ) ) {

		g_object_set( G_OBJECT( cvs ),
			      "root", root,
			      "base", base, NULL );
	
		screem_cvs_add( cvs, pathname, flags, log );

		g_free( log );
		g_free( flags );

		if( troot ) {
			g_free( *troot );
		}
		if( tmodule ) {
			g_free( *tmodule );
		}
		if( tbase ) {
			g_free( *tbase );
		}
	} 

	if( temp ) {
		g_free( temp );
	}
}

static void
cvs_menu_login_callback( BonoboUIComponent *component,
			       gpointer user_data,
			       const char *verb )
{
	ScreemWindow *window;
	ScreemCVS *cvs;

	gchar *root;
	gchar *flags;

	window = SCREEM_WINDOW( user_data );
	cvs = window->details->cvs;

	root = (gchar*)g_getenv( "CVSROOT" );
	if( ! root ) {
		root = "";
	}
	flags = "";

	if( screem_cvs_dialog( window, NULL, 
			       &root, NULL, NULL, &flags ) ) {

		g_object_set( G_OBJECT( cvs ),
			      "root", root, NULL );
	
		//	screem_cvs_import( cvs, flags, log, module );

		g_free( root );
		g_free( flags );
	}
}

static void
help_menu_help_callback( BonoboUIComponent *component,
			 gpointer user_data,
			 const char *verb )
{
	GError *error;
	ScreemWindow *window;

	window = SCREEM_WINDOW( user_data );

	error = NULL;

	gnome_help_display( "screem", NULL, &error );

	if (error) {
		GtkWidget *dialog;
		
		dialog = gtk_message_dialog_new (GTK_WINDOW (window),
						 0,
						 GTK_MESSAGE_ERROR,
						 GTK_BUTTONS_CLOSE,
						 _("Could not display help for Screem.\n"
						   "%s"),
						 error->message);
		
		g_signal_connect_swapped (dialog, "response",
					  G_CALLBACK (gtk_widget_destroy),
					  dialog);
		gtk_widget_show (dialog);
		
		g_error_free (error);
	}
}

static void
help_menu_about_screem_callback( BonoboUIComponent *component,
				   gpointer user_data,
				   const char *verb )
{
	GtkWidget *about;

	const gchar *COPYRIGHT = "1999-2002 David A Knight";
	const gchar *AUTHORS[] = {
		"David A Knight - Main Author",
		"Lee Mallabone - Sitecopy Integration / General ",
		"Joe Orton - Sitecopy",
		"Sven Liessem - General",
		"Matt Colyer - General",
		NULL
	};
	const gchar *DESCRIPTION = _( "\
SCREEM (Site CReation and Editing EnvironMent)\n\
Screem provides an integrated development environment for creating and maintaining websites\n\
SCREEM is licensed under the GNU General Public License version 2 or above" );
        GdkPixbuf *logo = NULL;
	const gchar *DOCUMENTERS[] = {
		"Matt Colyer",
		"David A Knight",
		NULL
	};
	const gchar *TRANSLATERS = _( "Translater Credits" );

	logo = gdk_pixbuf_new_from_file( SPLASH_DIR"/splash.png", NULL );

	about = gnome_about_new( PACKAGE, VERSION,
				 COPYRIGHT,
				 DESCRIPTION,
				 AUTHORS,
				 DOCUMENTERS,
				 TRANSLATERS,
				 logo );
	gtk_window_set_transient_for( GTK_WINDOW( about ),
				      GTK_WINDOW( user_data ) );
	gtk_dialog_run( GTK_DIALOG( about ) );
}

static void help_menu_tip_callback( BonoboUIComponent *component,
				    gpointer user_data,
				    const char *verb )
{
	ScreemWindow *window;
	ScreemApplication *application;

	window = SCREEM_WINDOW( user_data );
	application = SCREEM_APPLICATION( window->application );

	gtk_widget_show( application->hint );
}

/* Site view menu: FIXME doesn't belong here */

static void create_dir_callback( gchar *string, gpointer data )
{
	ScreemWindow *window;
	ScreemSite *site;
	GtkWidget *widget;

	window = SCREEM_WINDOW( data );
	site = screem_window_get_current( window );

	widget = GTK_WIDGET( g_object_get_data( G_OBJECT( window ),
						"siteview" ) );

	if( ! string ) {
                screem_window_show_message( window,
					    _( "No directory name specified." ) );
	} else if( ! screem_site_view_create_dir( site, widget, string ) ) {
                screem_window_show_message( window,
					    _( "Directory could not be created." ) );
        }
}

static void
site_view_menu_new_dir_callback( BonoboUIComponent *component,
				 gpointer user_data,
				 const char *verb )
{
	GtkWidget *widget;

	widget = gnome_request_dialog( FALSE, 
				       _( "Enter the directory name to create:" ),
				       "", 255, 
				       (GnomeStringCallback)create_dir_callback,
				       user_data, NULL );
	gtk_window_set_title( GTK_WINDOW( widget ), "Create Directory: - Screem" );
}

static void
site_view_menu_delete_file_callback( BonoboUIComponent *component,
				     gpointer user_data,
				     const char *verb )
{
	ScreemWindow *window;
	ScreemWindowDetails *details;
	ScreemSite *site;
	GtkWidget *view;
	GtkTreeModel *model;
	GtkTreeSelection *selection;
	GValue value = {0,};
	ScreemSiteViewNodeInfo *info;
	GtkTreeIter it;

	window = SCREEM_WINDOW( user_data );
	details = window->details;

	site = screem_window_get_current( window );
	view = gtk_notebook_get_nth_page( GTK_NOTEBOOK( details->file_view ),
					  0 );
	view = gtk_bin_get_child( GTK_BIN( view ) );
	selection = gtk_tree_view_get_selection( GTK_TREE_VIEW( view ) );
	
	gtk_tree_selection_get_selected( selection, &model, &it );
	
	gtk_tree_model_get_value( model, &it, FILE_BROWSER_DATA_COL,
				  &value );
	
	info = g_value_get_pointer( &value );
	if( info ) {
		/* delete info fullname */
		GtkWidget *confirm;
		gint button;
		
		confirm = gtk_message_dialog_new( NULL,
						  GTK_DIALOG_MODAL,
						  GTK_MESSAGE_WARNING,
						  GTK_BUTTONS_YES_NO,
						  _( "Really delete %s?" ),
						  info->fullname );
		gtk_window_set_wmclass( GTK_WINDOW( confirm ),
					"Screem",
					"delete_confirm_prompt" );
		gtk_window_set_transient_for( GTK_WINDOW( confirm ),
					      GTK_WINDOW( window ) );
		button = gtk_dialog_run( GTK_DIALOG( confirm ) );
		gtk_widget_destroy( confirm );
		if( button == GTK_RESPONSE_YES ) {
			delete_dir( info->fullname );
		}
	}
	g_value_unset( &value );
}

static void
site_view_menu_file_props_callback( BonoboUIComponent *component,
				    gpointer user_data,
				    const char *verb )
{
	ScreemWindow *window;
	ScreemWindowDetails *details;
	ScreemSite *site;
	GtkTreeModel *model;
	GtkWidget *view;
	GtkTreeSelection *selection;
	GValue value = {0,};
	ScreemSiteViewNodeInfo *info;
	GtkTreeIter it;

	window = SCREEM_WINDOW( user_data );
	details = window->details;

	site = screem_window_get_current( window );
	view = gtk_notebook_get_nth_page( GTK_NOTEBOOK( details->file_view ),
					  0 );
	view = GTK_WIDGET( g_object_get_data( G_OBJECT( window ), 
						"siteview" ) );
	selection = gtk_tree_view_get_selection( GTK_TREE_VIEW( view ) );

	gtk_tree_selection_get_selected( selection, &model, &it );
	
	gtk_tree_model_get_value( model, &it, FILE_BROWSER_DATA_COL, &value );
	
	info = g_value_get_pointer( &value );
	if( info ) {
		/* display info on info fullname */
		screem_site_view_show_properties( info,
						  GTK_WINDOW( window ) );
	}
	g_value_unset( &value );
}

static void
site_view_menu_exclude_callback( BonoboUIComponent *component,
				 gpointer user_data,
				 const char *verb )
{
	
}

static void
site_view_menu_ignore_callback( BonoboUIComponent *component,
				gpointer user_data,
				const char *verb )
{
	
}

static void
site_view_menu_ascii_callback( BonoboUIComponent *component,
			       gpointer user_data,
			       const char *verb )
{
	
}

void screem_window_recent_page_cb( GeditRecent *recent, const gchar *uri,
				   gpointer data )
{
	ScreemWindow *window;
	ScreemApplication *application;
	ScreemSite *site;
	ScreemPage *page;

	window = SCREEM_WINDOW( data );
	application = SCREEM_APPLICATION( window->application );
	site = screem_application_get_default_site( application );

	page = screem_page_new();
	screem_page_set_pathname( page, uri );

	if( ! screem_site_add_page( site, page ) ) {
		/* already open in the site, FIXME: what now? */
	}

	screem_window_set_current( window, site );
	screem_window_set_document( window, page );
}

void screem_window_recent_site_cb( GeditRecent *recent, const gchar *uri,
				   gpointer data )
{
	ScreemWindow *window;

	window = SCREEM_WINDOW( data );

	screem_site_open_with_filename( window, uri );
}


void
screem_window_initialise_menus( ScreemWindow *window )
{
	BonoboUIVerb verbs[] = {
		/* file menu */

		/* file->new */
		BONOBO_UI_VERB( "New Window", file_menu_new_window_callback ),
		BONOBO_UI_VERB( "New Site", file_menu_new_site_callback ),
		BONOBO_UI_VERB( "New Document", file_menu_new_document_callback ),
		BONOBO_UI_VERB( "New Blank Document", file_menu_new_blank_document_callback ),
		BONOBO_UI_VERB( "New Site Template", file_menu_new_site_template_callback ),

		BONOBO_UI_VERB( "Open Document", file_menu_open_document_callback ),
		BONOBO_UI_VERB( "Save Document", file_menu_save_document_callback ),
		BONOBO_UI_VERB( "Save Document As", file_menu_save_document_as_callback ),
		BONOBO_UI_VERB( "Close Document", file_menu_close_document_callback ),

		BONOBO_UI_VERB( "Open Site", file_menu_open_site_callback ),
		BONOBO_UI_VERB( "Save All", file_menu_save_all_callback ),
		BONOBO_UI_VERB( "Close Site", file_menu_close_site_callback ),


		BONOBO_UI_VERB( "Print", file_menu_print_callback ),
		BONOBO_UI_VERB( "PrintPreview", file_menu_print_preview_callback ),
		BONOBO_UI_VERB( "Offline", file_menu_offline_callback ),
		BONOBO_UI_VERB( "Online", file_menu_offline_callback ),

		BONOBO_UI_VERB( "Exit", file_menu_exit_application_callback ),

		/* Edit Menu */
		BONOBO_UI_VERB( "Undo", edit_menu_undo_callback ),
		BONOBO_UI_VERB( "Redo", edit_menu_redo_callback ),

		BONOBO_UI_VERB( "Cut", edit_menu_cut_callback ),
		BONOBO_UI_VERB( "Copy", edit_menu_copy_callback ),
		BONOBO_UI_VERB( "Paste", edit_menu_paste_callback ),
		BONOBO_UI_VERB( "PasteEncoded", edit_menu_paste_enc_callback ),
		BONOBO_UI_VERB( "Clear", edit_menu_clear_callback ),
		BONOBO_UI_VERB( "SelectAll", edit_menu_select_all_callback ),
		BONOBO_UI_VERB( "SelectContext", 
				edit_menu_select_context_callback ),

		BONOBO_UI_VERB( "MarkEditable",
				edit_menu_mark_editable_callback ),

		BONOBO_UI_VERB( "Search", edit_menu_search_callback ),
		BONOBO_UI_VERB( "Replace", edit_menu_search_callback ),

		BONOBO_UI_VERB( "Bookmarks", edit_menu_bookmarks_callback ),

		/* View Menu */
		BONOBO_UI_VERB( "SwitchEditor", view_menu_editor_callback ),
		BONOBO_UI_VERB( "SwitchPreview", view_menu_preview_callback ),
		BONOBO_UI_VERB( "SwitchLinkView",view_menu_linkview_callback ),
		BONOBO_UI_VERB( "SwitchTreeView",view_menu_treeview_callback ),

		/* Insert Menu (most dealt with in toolbars / wizards) */
		BONOBO_UI_VERB( "InsertDoctype",insert_menu_doctype_callback ),
		BONOBO_UI_VERB( "InsertFile", insert_menu_file_callback ),

		/* Tools menu */
		BONOBO_UI_VERB( "SiteSettings", 
				tool_menu_site_settings_callback ),
		BONOBO_UI_VERB( "Preferences", 
				tool_menu_preferences_callback ),

		/* CVS Menu */
		BONOBO_UI_VERB( "CheckoutSite", 
				cvs_menu_checkout_site_callback ),
		BONOBO_UI_VERB( "UpdateSite", cvs_menu_update_site_callback ),
		BONOBO_UI_VERB( "UpdatePage", cvs_menu_update_page_callback ),
		BONOBO_UI_VERB( "ImportSite", cvs_menu_import_site_callback ),
		BONOBO_UI_VERB( "CommitSite", cvs_menu_commit_site_callback ),
		BONOBO_UI_VERB( "CommitPage", cvs_menu_commit_page_callback ),
		BONOBO_UI_VERB( "AddPage", cvs_menu_add_page_callback ),

		BONOBO_UI_VERB( "UpdateFile", cvs_menu_update_page_callback ),
		BONOBO_UI_VERB( "CommitFile", cvs_menu_commit_page_callback ),
		BONOBO_UI_VERB( "AddFile", cvs_menu_add_page_callback ),

		BONOBO_UI_VERB( "CVSLogin", cvs_menu_login_callback ),

		/* help menu */
		BONOBO_UI_VERB( "Help",
				help_menu_help_callback ),
		BONOBO_UI_VERB( "About Screem",
				help_menu_about_screem_callback ),
		BONOBO_UI_VERB( "Tip Of The Day", help_menu_tip_callback ),

		/* site view menu */
		BONOBO_UI_VERB( "New Directory",
				site_view_menu_new_dir_callback ),
		BONOBO_UI_VERB( "Delete File",
				site_view_menu_delete_file_callback ),
		BONOBO_UI_VERB( "File Properties", 
				site_view_menu_file_props_callback ),
		BONOBO_UI_VERB( "ExcludeFlag",
				site_view_menu_exclude_callback ),
		BONOBO_UI_VERB( "IgnoreFlag", site_view_menu_ignore_callback ),
		BONOBO_UI_VERB( "ASCIIFlag", site_view_menu_ascii_callback ),

		BONOBO_UI_VERB_END
	};

	bonobo_ui_component_freeze( window->details->shell_ui, NULL );

	bonobo_ui_component_add_verb_list_with_data( window->details->shell_ui,
						     verbs, window );

	bonobo_ui_component_thaw( window->details->shell_ui, NULL );
}
